import MuseScore 3.0
import QtQuick 2.0

MuseScore {
    version: "1.0";
    description: "Replaces chord symbol tails"
    menuPath: "Plugins.ReplaceChordSyms."

    id: replaceChordSyms

    Component.onCompleted : {
        if (mscoreMajorVersion >= 4) {
            replaceChordSyms.title = "Replace Chord Symbols"
        }
     }


    function setCursorToTime(cursor, time) {
        cursor.rewind(0);
        while (cursor.segment) {
            var current_time = cursor.tick;
            if (current_time >= time) {
                return true;
            }
            cursor.next();
        }
        cursor.rewind(0);
        return false;
    }

    function findheadlen(harmony){								
		var str = harmony;							
		var t = str.substr(1,1);							
		if ((t == "b") || (t == "#")){							
			return 2;						
		}							
		else{							
			return 1;						
		}							
	}
    
    function findtail(str){
        var notail = "";
        var headlen = findheadlen(str);					
        var hstr = str.substr(0,headlen);					
        if (str.length - headlen > 0) {
            var tail = str.substr(headlen,(str.length - headlen))					
            return tail;
        }
        else
            return notail;
    }

    function replaceChordSymbols(cursor, segment) {
        var replace = "Dm7";
        var rwith = "D-";

        var rtail = findtail(replace)
        var wtail = findtail(rwith)
        //console.log ("rtail = ", rtail)
        //console.log ("wtail = ", wtail)
        
        while (segment) {
            var aCount = 0;
            var annotation = segment.annotations[aCount];

            while (annotation) {
                if (annotation.type == Element.HARMONY) {
                    var harmony = annotation;
                    var str = harmony.text;
                    //console.log ('str = ', str);				
                    if (str != ""){
                        // slash chord?
                        var n = str.search("/");					
                        if (n > -1){
                            //there may be tails to fix in both main & slash chord
                            //first fix the main head
                            var headstr = str.substr(0,n);
                            //console.log ('headstr = ', headstr);		
                            var headlen = findheadlen(str);					
                            var head = str.substr(0,headlen);
                            if (str.length - headlen > 0) {
                                var tail = str.substr(headlen,(n - headlen))					
                        	    //console.log ('75 tail = ', tail);
                                if(tail == rtail) headstr = head + wtail;    	
                            }
                            					    
                            var slashstr = 	str.substr(n+1)	;		
                            //console.log('slashstr = ' + slashstr);			
                            str = str.substr(0,n);	
                            var slashlen = 	findheadlen(slashstr);
                            var slashhead = slashstr.substr(0,slashlen); 	
                            if (slashstr.length - slashlen > 0){
                                var slashtail = slashstr.substr(slashlen,(slashstr.length - slashlen))					
                                //console.log ('slashhead = ', slashhead);				
                                //console.log ('slashtail = ', slashtail);
                                if(slashtail == rtail) {
                                    //newharm = head + rwith;
                                    removeElement (harmony);
                                    harmony = newElement(Element.HARMONY);
                                    harmony.text = headstr + "/" + slashhead + wtail;
                                    cursor.rewindToTick(segment.tick);
                                    cursor.add(harmony);
                                    console.log ("Replacing " + headstr + "/" + slashhead + slashtail + " with " + headstr + "/" + slashhead + wtail)
                                } // slashtail == rtail
                            } // end slashstr.length - slashlen > 0  		
                        } // end n > -1	
                        else {
                            var headlen = findheadlen(str);					
                            var head = str.substr(0,headlen);					
                                
                            //console.log ('head = ', head);	
                                                        
                            if (str.length - headlen > 0) {
                                var tail = str.substr(headlen,(str.length - headlen))					
                                //console.log ('tail = ', tail);	
                                if(tail == rtail) {
                                    //newharm = head + rwith;
                                    removeElement (harmony);
                                    harmony = newElement(Element.HARMONY);
                                    harmony.text = head + wtail;
                                    cursor.rewindToTick(segment.tick);
                                    cursor.add(harmony);
                                    console.log ("Replacing " + str + " with " + head + wtail)
                                } // if(tail == rtail)
                            } // end str.length - headlen > 0)
                        } // end else			
                    }// end if str
                }// end annotation.type == Element.HARMONY
                annotation = segment.annotations[++aCount];
            }
            segment = segment.next;
        } // end while segment

    } // end function

function smartQuit() {
		
		if (mscoreMajorVersion < 4) {Qt.quit()}
		else {quit()}
	}//smartQuit
    

    onRun: {
        if (typeof curScore === 'undefined') {
            smartQuit();
        }
        
        var cursor = curScore.newCursor();
        cursor.rewind(0); // beginning of score									
    
        var segment = curScore.firstSegment();
        if (mscoreMajorVersion < 4) {curScore.startCmd();}
        else {curScore.cmdStart();}

        replaceChordSymbols(cursor, segment)
        
        if (mscoreMajorVersion < 4) {curScore.endCmd();}
        else { curScore.cmdEnd();}
        
        
        smartQuit()    
        } // end onRun					
}
